{% extends "email/layout.pt_pt.html.tpl" %}
{% block title %}Feliz Aniversário{% endblock %}
{% block content %}
    <p>
        A equipa de gestão deseja-lhe um feliz aniversário!
    </p>
{% endblock %}
