{% extends "partials/layout.html.tpl" %}
{% block title %}Home{% endblock %}
{% block name %}Omnix{% endblock %}
{% block content %}
    <div class="quote">
        Omnix is a simple web application for minimal control of an omni instance.
        This beast is written <strong>only in python</strong>.
    </div>
{% endblock %}
