<!-- css inclusion -->
<link rel="stylesheet" type="text/css" href="//libs.bemisc.com/uxf/css/ux-min.css" />
<link rel="stylesheet" type="text/css" href="//libs.bemisc.com/layout/css/layout.flat.css" />
<link rel="stylesheet" type="text/css" href="//libs.bemisc.com/layout/css/layout.extras.css" />
<link rel="stylesheet" type="text/css" href="{{ url_for('static', filename = 'css/layout.css') }}" />

<!-- favicon inclusion -->
<link rel="shortcut icon" href="{{ url_for('static', filename = 'images/favicon.ico') }}" />

<!-- javascript inclusion -->
<script type="text/javascript" src="//ajax.googleapis.com/ajax/libs/jquery/1.5.1/jquery.min.js"></script>
<script type="text/javascript" src="//libs.bemisc.com/uxf/js/ux-min.js"></script>
<script type="text/javascript" src="//libs.bemisc.com/layout/js/layout-min.js"></script>
<script type="text/javascript" src="{{ url_for('static', filename = 'js/main.js') }}"></script>
